#---------------- csv_get_rec -----------------
#str -> "f1","f2", ... ,"fn"
#arr_res[1]=f1; arr_res[2]=f2; ... arr_res[n]=fn;
#return fields number   
#----------------------------------------------
function csv_get_rec(str,arr_res,    n){
  n = split(str,arr_res, "\",\"");
  gsub(/^"/,"",arr_res[1]);
  gsub(/"$/,"",arr_res[n]);
  return n;
}

#---------------- csv_make_rec -----------------
#arr_fld[1]=f1; arr_fld[2]=f2; ... arr_res[n_fld]=fn_fld;
#return -> "f1","f2", ... ,"fn_fld"
#-----------------------------------------------
function csv_make_rec(arr_fld,n_fld,   i,s){
  s = "\"" arr_fld[1] "\"";
  for(i=2; i<=n_fld; i++)
    s = s "," "\"" arr_fld[i] "\"";
  return s;
}

#---------------- csv_newsep -----------------
#str -> "f1","f2", ... ,"fn"
#return f1 sep f2 sep fn   
#---------------------------------------------
function csv_newsep(str,sep,    n,tmp_a,i,s){
  n = csv_get_rec(str,tmp_a);
  s = tmp_a[1]
  for(i=2; i<=n; i++) s = s sep tmp_a[i];
  return s;
}

#---------------- tbl_sort_arr -----------------
#arr - array to sort
#nr  - number of elements
#stp=="DESC" - descending sort
#stp!="DESC" - ascending sort
#return 1 - nothing to sort
#       0 - OK
#-----------------------------------------------
function tbl_sort_arr(arr,nr,stp,  i,j,v){
  if(nr<=1) return 1;
  for(i=1; i<nr; i++)
    for(j=i+1; j<=nr; j++)
      if(((stp=="DESC") && (arr[i]<arr[j])) || 
         ((stp!="DESC") && (arr[i]>arr[j])) ){
        v = arr[i];
        arr[i] = arr[j];
        arr[j] = v;
      }
  return 0;
}

#---------------- tbl_new -----------------
#            create a new table
#d_data[key] = record (now empty)
#s_key - key of table
#s_fld - fields list (f1#f2#...#fn)
#a_ind - for record numbers and serv info
#a_ind["rec_total"] - total records count (now 0)
#a_ind["fld_list"]  = s_fld
#a_ind["fld_total"] - total fields count
#a_ind["key_index"] - number of key position in fields list
#a_ind["f"] - number of field "f" position in fields list
#return 1 - no key in fields list 
#       0 - OK
#-------------------------------------------
function tbl_new(a_ind, a_data, s_fld, s_key,    tmp_a,i,n,k){
  delete a_ind;
  delete a_data;
  a_ind["rec_total"] = 0;
  a_ind["fld_list"] = s_fld; 
  n = split(s_fld,tmp_a,"#");
  if(n==0) return 1;
  for(i=1; i<=n; i++)
    a_ind[tmp_a[i]] = i;
  a_ind["fld_total"] = n;
  k = 0; 
  for(i=1; i<=n; i++)
    if(s_key == tmp_a[i]) k = i;
  if(k==0) return 1;
  a_ind["key_index"] = k;
  return 0;
}

#---------------- tbl_put_rec -----------------
#            put a record into the table
#s_key - key of record
#a_val - array of fields values
#a_data[s_key] = string of fields values
#return 1 - record with key "s_key" inserted
#       0 - record with key "s_key" existed and updated
#----------------------------------------------
function tbl_put_rec(a_ind, a_data, s_key, a_val,   s_data){
  a_val[a_ind["key_index"]] = s_key;
  s_data = csv_make_rec(a_val,a_ind["fld_total"]);
  if(s_key in a_data){
    a_data[s_key] = s_data;
    return 0;
  }
  a_data[s_key] = s_data;
  a_ind[++a_ind["rec_total"]] = s_key;
  return 1;
}

#---------------- tbl_get_rec -----------------
#            get a record from the table
#s_key - key of record
#a_val - array of fields values
#a_data[s_key] = string of fields values
#return 1 - record with key "s_key" new, only key value put into a_val
#       0 - record with key "s_key" existed, fields values put into a_val
#----------------------------------------------
function tbl_get_rec(a_ind, a_data, s_key, a_val,   i){
  if(s_key in a_data){
    csv_get_rec(a_data[s_key],a_val);
    return 0;
  }
  delete a_val;
  for(i=1; i<=a_ind["fld_total"]; i++) a_val[i] = "";
  a_val[a_ind["key_index"]] = s_key;
  return 1;
}

#---------------- tbl_read -----------------
#            read a table from file
# 1th record in file must be a csv string with fields list
#            new table created !!!
#fn - file name
#s_key - name of key
#return 1 - no key in fields list 
#       0 - OK
#-------------------------------------------
function tbl_read(a_ind, a_data, s_key, fn,    s,tmp_a,i,n,r){
  r = getline s < fn;
  if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
  s = csv_newsep(s ,"#");
  if(tbl_new(a_ind, a_data, s, s_key)) return 1;
  while(r = getline s < fn){
    if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
    n = csv_get_rec(s,tmp_a);
    if(n < a_ind["key_index"]) return 1;
    tbl_put_rec(a_ind, a_data, tmp_a[a_ind["key_index"]], tmp_a);
  }
  close(fn);
  return 0;
}

#---------------- tbl_print -----------------
#            print a table to file
# 1th record in file is csv string with fields list
#fn - file name (if fn=="" - print to output stream)
#return - nothing
#--------------------------------------------
function tbl_print(a_ind, a_data,fn,    n,tmp_a,s){
  n = split(a_ind["fld_list"],tmp_a,"#");
  s = csv_make_rec(tmp_a,n);
  if(fn != "") print s > fn;
          else print s;
  for(n=1; n<=a_ind["rec_total"]; n++)
    if(fn != "") print a_data[a_ind[n]] > fn;
            else print a_data[a_ind[n]];
}

#---------------- tbl_sort_fld -----------------
#      sort information in an individual field 
#  field must contain subfields separated with string sep
#                example (sep = "#"):    
#    2011-03-29 52#2011-03-29 51#2009-03-30 85
#s_fld - field name
#sep   - separator
#return - nothing
#-----------------------------------------------
function tbl_sort_fld(a_ind,a_data,s_fld,sep,tp,    tmp_a,n,s,i){
  for(i=1; i<=a_ind["rec_total"]; i++){
    tbl_get_rec(a_ind,a_data,a_ind[i],tmp_a);
    s = tmp_a[a_ind[s_fld]];
    n = split(s,arr,sep);
    tbl_sort_arr(arr,n,tp);
    s = arr[1];
    for(j=2; j<=n; j++) s = s sep arr[j];
    tmp_a[a_ind[s_fld]] = s;
    tbl_put_rec(a_ind,a_data,a_ind[i],tmp_a);
  }
}

#---------------- tbl_sort ---------------------
#        sort information in a table 
#
#s_fld - fields list in string (separator -"#")
#s_stp - sort types in string (separator -"#")
#stp=="DESC" - descending sort
#stp!="DESC" - ascending sort
#return 1 - nothing to sort
#       0 - OK
#-----------------------------------------------
function tbl_sort(a_ind,a_data,s_fld,s_stp,    
                  pri,a_fld,a_stp,k,n,ai,aj,nr,i,j,si,sj,v){
  n = split(s_fld,a_fld,"#");
  split(s_stp,a_stp,"#");
  nr = a_ind["rec_total"];
  if(nr<=1) return 1;
  for(i=1; i<nr; i++)
    for(j=i+1; j<=nr; j++){
      tbl_get_rec(a_ind,a_data,a_ind[i],ai);
      tbl_get_rec(a_ind,a_data,a_ind[j],aj);
      pri = 0;
      for(k=1; k<=n; k++){
        si = ai[a_ind[a_fld[k]]];
        sj = aj[a_ind[a_fld[k]]];
        if(si == sj) continue;
        if(((a_stp[k]=="DESC") && (si<sj)) || 
           ((a_stp[k]!="DESC") && (si>sj)) )
          pri = 1;
        break;
      }
      if(pri){
        v = a_ind[i];
        a_ind[i] = a_ind[j];
        a_ind[j] = v;
      }
    }
  return 0;
}

#---------------- tbl_debug_print -----------------
#            debug print a table to file
#fn - file name (if fn=="" - print to output stream)
#return - nothing
#--------------------------------------------------
function tbl_debug_print(a_ind, a_data,    i){
  for(i in a_ind)
    if(i !~ /^[0-9]/)
      if (fn != "") printf("a_ind[%s]=%s\n",i,a_ind[i]) > fn
               else printf("a_ind[%s]=%s\n",i,a_ind[i]);
  if (fn != "") print "-------------------------"  > fn
           else print "-------------------------";
  for(i=1; i<=a_ind["rec_total"]; i++)
    if (fn != "") printf("[%s]=%s ->%s\n",i,a_ind[i],a_data[a_ind[i]]) > fn
             else printf("[%s]=%s ->%s\n",i,a_ind[i],a_data[a_ind[i]]);
}


BEGIN{
#KEY = US_ID "_" US_CAT
  tbl_new(indUS, dataUS, "KEY#US_ID#US_NM#US_CAT#CAT_NM", "KEY");
  
# "ID","VS","CT ID","FNM","SNM"  
  tbl_read(indCR,dataCR,"ID","man_course/cr.csv");

#"ID","PARENT","NAME","DEPTH","PATH","NUM CR"
  tbl_read(indCAT,dataCAT,"ID","baza/cr_cat.csv");
}

NR == 1{
  next;
}

#"US ID","lastname","firstname","CR ID","LAST AC","fullname","category"
#  1         2           3          4       5         6           7

{
  csv_get_rec($0,arr_line)
  us_id = arr_line[1];
  cr_id = arr_line[4];
  us_nm = arr_line[2] " " arr_line[3];
  cr_nm = arr_line[6];

#KEY = US_ID "_" US_CAT
# tbl_new(indUS, dataUS, "KEY#US_ID#US_NM#US_CAT#CAT_NM", "KEY");
  r = tbl_get_rec(indCR,dataCR,cr_id,tmp_a);
  cat_id = tmp_a[indCR["CT ID"]];
  tbl_get_rec(indCAT,dataCAT,cat_id,tmp_a);
  cat_nm = tmp_a[indCAT["NAME"]];
  
  key = us_id "_" cat_id;
 
  delete tmp_a;
  tbl_get_rec(indUS,dataUS,key,tmp_a);
  tmp_a[indUS["US_ID"]] = us_id;
  tmp_a[indUS["US_NM"]] = us_nm;
  tmp_a[indUS["US_CAT"]] = cat_id;
  tmp_a[indUS["CAT_NM"]] = cat_nm;
  tbl_put_rec(indUS,dataUS,key,tmp_a);
}

END{
  tbl_sort(indUS,dataUS,"US_ID#US_CAT","ASC#ASC");
  tbl_print(indUS,dataUS,"man_kaf/tch_kaf_to_hand.csv");

#  debug_tbl_print(indUS,dataUS);
}
