#---------------- csv_get_rec -----------------
#str -> "f1","f2", ... ,"fn"
#arr_res[1]=f1; arr_res[2]=f2; ... arr_res[n]=fn;
#return fields number   
#----------------------------------------------
function csv_get_rec(str,arr_res,    n){
  n = split(str,arr_res, "\",\"");
  gsub(/^"/,"",arr_res[1]);
  gsub(/"$/,"",arr_res[n]);
  return n;
}

#---------------- csv_make_rec -----------------
#arr_fld[1]=f1; arr_fld[2]=f2; ... arr_res[n_fld]=fn_fld;
#return -> "f1","f2", ... ,"fn_fld"
#-----------------------------------------------
function csv_make_rec(arr_fld,n_fld,   i,s){
  s = "\"" arr_fld[1] "\"";
  for(i=2; i<=n_fld; i++)
    s = s "," "\"" arr_fld[i] "\"";
  return s;
}

#---------------- csv_newsep -----------------
#str -> "f1","f2", ... ,"fn"
#return f1 sep f2 sep fn   
#---------------------------------------------
function csv_newsep(str,sep,    n,tmp_a,i,s){
  n = csv_get_rec(str,tmp_a);
  s = tmp_a[1]
  for(i=2; i<=n; i++) s = s sep tmp_a[i];
  return s;
}

#---------------- tbl_sort_arr -----------------
#arr - array to sort
#nr  - number of elements
#stp=="DESC" - descending sort
#stp!="DESC" - ascending sort
#return 1 - nothing to sort
#       0 - OK
#-----------------------------------------------
function tbl_sort_arr(arr,nr,stp,  i,j,v){
  if(nr<=1) return 1;
  for(i=1; i<nr; i++)
    for(j=i+1; j<=nr; j++)
      if(((stp=="DESC") && (arr[i]<arr[j])) || 
         ((stp!="DESC") && (arr[i]>arr[j])) ){
        v = arr[i];
        arr[i] = arr[j];
        arr[j] = v;
      }
  return 0;
}

#---------------- tbl_new -----------------
#            create a new table
#d_data[key] = record (now empty)
#s_key - key of table
#s_fld - fields list (f1#f2#...#fn)
#a_ind - for record numbers and serv info
#a_ind["rec_total"] - total records count (now 0)
#a_ind["fld_list"]  = s_fld
#a_ind["fld_total"] - total fields count
#a_ind["key_index"] - number of key position in fields list
#a_ind["f"] - number of field "f" position in fields list
#return 1 - no key in fields list 
#       0 - OK
#-------------------------------------------
function tbl_new(a_ind, a_data, s_fld, s_key,    tmp_a,i,n,k){
  delete a_ind;
  delete a_data;
  a_ind["rec_total"] = 0;
  a_ind["fld_list"] = s_fld; 
  n = split(s_fld,tmp_a,"#");
  if(n==0) return 1;
  for(i=1; i<=n; i++)
    a_ind[tmp_a[i]] = i;
  a_ind["fld_total"] = n;
  k = 0; 
  for(i=1; i<=n; i++)
    if(s_key == tmp_a[i]) k = i;
  if(k==0) return 1;
  a_ind["key_index"] = k;
  return 0;
}

#---------------- tbl_put_rec -----------------
#            put a record into the table
#s_key - key of record
#a_val - array of fields values
#a_data[s_key] = string of fields values
#return 1 - record with key "s_key" inserted
#       0 - record with key "s_key" existed and updated
#----------------------------------------------
function tbl_put_rec(a_ind, a_data, s_key, a_val,   s_data){
  a_val[a_ind["key_index"]] = s_key;
  s_data = csv_make_rec(a_val,a_ind["fld_total"]);
  if(s_key in a_data){
    a_data[s_key] = s_data;
    return 0;
  }
  a_data[s_key] = s_data;
  a_ind[++a_ind["rec_total"]] = s_key;
  return 1;
}

#---------------- tbl_get_rec -----------------
#            get a record from the table
#s_key - key of record
#a_val - array of fields values
#a_data[s_key] = string of fields values
#return 1 - record with key "s_key" new, only key value put into a_val
#       0 - record with key "s_key" existed, fields values put into a_val
#----------------------------------------------
function tbl_get_rec(a_ind, a_data, s_key, a_val,   i){
  if(s_key in a_data){
    csv_get_rec(a_data[s_key],a_val);
    return 0;
  }
  delete a_val;
  for(i=1; i<=a_ind["fld_total"]; i++) a_val[i] = "";
  a_val[a_ind["key_index"]] = s_key;
  return 1;
}

#---------------- tbl_read -----------------
#            read a table from file
# 1th record in file must be a csv string with fields list
#            new table created !!!
#fn - file name
#s_key - name of key
#return 1 - no key in fields list 
#       0 - OK
#-------------------------------------------
function tbl_read(a_ind, a_data, s_key, fn,    s,tmp_a,i,n,r){
  r = getline s < fn;
  if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
  s = csv_newsep(s ,"#");
  if(tbl_new(a_ind, a_data, s, s_key)) return 1;
  while(r = getline s < fn){
    if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
    n = csv_get_rec(s,tmp_a);
    if(n < a_ind["key_index"]) return 1;
    tbl_put_rec(a_ind, a_data, tmp_a[a_ind["key_index"]], tmp_a);
  }
  close(fn);
  return 0;
}

#---------------- tbl_print -----------------
#            print a table to file
# 1th record in file is csv string with fields list
#fn - file name (if fn=="" - print to output stream)
#return - nothing
#--------------------------------------------
function tbl_print(a_ind, a_data,fn,    n,tmp_a,s){
  n = split(a_ind["fld_list"],tmp_a,"#");
  s = csv_make_rec(tmp_a,n);
  if(fn != "") print s > fn;
          else print s;
  for(n=1; n<=a_ind["rec_total"]; n++)
    if(fn != "") print a_data[a_ind[n]] > fn;
            else print a_data[a_ind[n]];
}

#---------------- tbl_sort_fld -----------------
#      sort information in an individual field 
#  field must contain subfields separated with string sep
#                example (sep = "#"):    
#    2011-03-29 52#2011-03-29 51#2009-03-30 85
#s_fld - field name
#sep   - separator
#return - nothing
#-----------------------------------------------
function tbl_sort_fld(a_ind,a_data,s_fld,sep,tp,    tmp_a,n,s,i){
  for(i=1; i<=a_ind["rec_total"]; i++){
    tbl_get_rec(a_ind,a_data,a_ind[i],tmp_a);
    s = tmp_a[a_ind[s_fld]];
    n = split(s,arr,sep);
    tbl_sort_arr(arr,n,tp);
    s = arr[1];
    for(j=2; j<=n; j++) s = s sep arr[j];
    tmp_a[a_ind[s_fld]] = s;
    tbl_put_rec(a_ind,a_data,a_ind[i],tmp_a);
  }
}

#---------------- tbl_sort ---------------------
#        sort information in a table 
#
#s_fld - fields list in string (separator -"#")
#s_stp - sort types in string (separator -"#")
#stp=="DESC" - descending sort
#stp!="DESC" - ascending sort
#return 1 - nothing to sort
#       0 - OK
#-----------------------------------------------
function tbl_sort(a_ind,a_data,s_fld,s_stp,    
                  pri,a_fld,a_stp,k,n,ai,aj,nr,i,j,si,sj,v){
  n = split(s_fld,a_fld,"#");
  split(s_stp,a_stp,"#");
  nr = a_ind["rec_total"];
  if(nr<=1) return 1;
  for(i=1; i<nr; i++)
    for(j=i+1; j<=nr; j++){
      tbl_get_rec(a_ind,a_data,a_ind[i],ai);
      tbl_get_rec(a_ind,a_data,a_ind[j],aj);
      pri = 0;
      for(k=1; k<=n; k++){
        si = ai[a_ind[a_fld[k]]];
        sj = aj[a_ind[a_fld[k]]];
        if(si == sj) continue;
        if(((a_stp[k]=="DESC") && (si<sj)) || 
           ((a_stp[k]!="DESC") && (si>sj)) )
          pri = 1;
        break;
      }
      if(pri){
        v = a_ind[i];
        a_ind[i] = a_ind[j];
        a_ind[j] = v;
      }
    }
  return 0;
}

#---------------- tbl_debug_print -----------------
#            debug print a table to file
#fn - file name (if fn=="" - print to output stream)
#return - nothing
#--------------------------------------------------
function tbl_debug_print(a_ind, a_data,    i){
  for(i in a_ind)
    if(i !~ /^[0-9]/)
      if (fn != "") printf("a_ind[%s]=%s\n",i,a_ind[i]) > fn
               else printf("a_ind[%s]=%s\n",i,a_ind[i]);
  if (fn != "") print "-------------------------"  > fn
           else print "-------------------------";
  for(i=1; i<=a_ind["rec_total"]; i++)
    if (fn != "") printf("[%s]=%s ->%s\n",i,a_ind[i],a_data[a_ind[i]]) > fn
             else printf("[%s]=%s ->%s\n",i,a_ind[i],a_data[a_ind[i]]);
}


#--------------------- read_strings  ------------------
#              read strings list from file
#fn - file name
#srt_arr - array for strings 
#return -1 if error
#------------------------------------------------------
function read_strings(fn,srt_arr,    s_key,s_val,r){
  while(r = getline s_key < fn){
    if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
    r = getline s_val < fn
    srt_arr[s_key] = s_val;
  }
  return 0;
}

BEGIN{
  read_strings("___str.___",GB_STR);

  tbl_new(indCR,dataCR,"CR_TP#CR_ID#CR_LA#CR_US#CR_ACT#CR_NM#CR_AD","CR_ID");
  
#-------- teacher last access
  tbl_read(indSRV,dataSRV,"CR_ID","crtchla.csv");
  for(i=1; i<=indSRV["rec_total"]; i++){
    s_key = indSRV[i];
    tbl_get_rec(indSRV,dataSRV,s_key,tmp_a);
    s = tmp_a[indSRV["CR_US"]];
    s = substr(s,1,10);
    tbl_get_rec(indCR,dataCR,s_key,tmp_a);
    tmp_a[indCR["CR_LA"]] = s;
    tbl_put_rec(indCR,dataCR,s_key,tmp_a);
  }

#-------- course name & visible
  tbl_read(indSRV,dataSRV,"ID","man_course/cr.csv");
  for(i=1; i<=indSRV["rec_total"]; i++){
    s_key = indSRV[i];
    tbl_get_rec(indSRV,dataSRV,s_key,tmp_a);
    s = tmp_a[indSRV["FNM"]];
    s1 = tmp_a[indSRV["VS"]];
    tbl_get_rec(indCR,dataCR,s_key,tmp_a);
    tmp_a[indCR["CR_NM"]] = s;
    tmp_a[indCR["CR_TP"]] = s1;
    tbl_put_rec(indCR,dataCR,s_key,tmp_a);
  }

#-------- users namber
  tbl_read(indSRV,dataSRV,"CR ID","baza/cr_la_cnt.csv");
  for(i=1; i<=indSRV["rec_total"]; i++){
    s_key = indSRV[i];
    tbl_get_rec(indSRV,dataSRV,s_key,tmp_a);
    s = tmp_a[indSRV["NUM US"]];
    tbl_get_rec(indCR,dataCR,s_key,tmp_a);
    tmp_a[indCR["CR_US"]] = s;
    tbl_put_rec(indCR,dataCR,s_key,tmp_a);
  }

#-------- users activity
  tbl_read(indSRV,dataSRV,"CR ID","baza/cr_act_cnt.csv");
  for(i=1; i<=indSRV["rec_total"]; i++){
    s_key = indSRV[i];
    tbl_get_rec(indSRV,dataSRV,s_key,tmp_a);
    s = tmp_a[indSRV["NUM ACT"]];
    tbl_get_rec(indCR,dataCR,s_key,tmp_a);
    tmp_a[indCR["CR_ACT"]] = s;
    tbl_put_rec(indCR,dataCR,s_key,tmp_a);
  }

#-------- course URL
  for(i=1; i<=indCR["rec_total"]; i++){
    s_key = indCR[i];
    tbl_get_rec(indCR,dataCR,s_key,tmp_a);
    GB_STR["COURSE_URL"]
    s = GB_STR["COURSE_URL"] s_key;
    tmp_a[indCR["CR_AD"]] = s;
    tbl_put_rec(indCR,dataCR,s_key,tmp_a);
  }

  tbl_print(indCR,dataCR,"man_course/cr_info_hand.csv");
  
#  debug_tbl_print(indCR,dataCR);
}
