#---------------- csv_get_rec -----------------
#str -> "f1","f2", ... ,"fn"
#arr_res[1]=f1; arr_res[2]=f2; ... arr_res[n]=fn;
#return fields number   
#----------------------------------------------
function csv_get_rec(str,arr_res,    n){
  n = split(str,arr_res, "\",\"");
  gsub(/^"/,"",arr_res[1]);
  gsub(/"$/,"",arr_res[n]);
  return n;
}

#---------------- csv_make_rec -----------------
#arr_fld[1]=f1; arr_fld[2]=f2; ... arr_res[n_fld]=fn_fld;
#return -> "f1","f2", ... ,"fn_fld"
#-----------------------------------------------
function csv_make_rec(arr_fld,n_fld,   i,s){
  s = "\"" arr_fld[1] "\"";
  for(i=2; i<=n_fld; i++)
    s = s "," "\"" arr_fld[i] "\"";
  return s;
}

#---------------- csv_newsep -----------------
#str -> "f1","f2", ... ,"fn"
#return f1 sep f2 sep fn   
#---------------------------------------------
function csv_newsep(str,sep,    n,tmp_a,i,s){
  n = csv_get_rec(str,tmp_a);
  s = tmp_a[1]
  for(i=2; i<=n; i++) s = s sep tmp_a[i];
  return s;
}

#---------------- tbl_sort_arr -----------------
#arr - array to sort
#nr  - number of elements
#stp=="DESC" - descending sort
#stp!="DESC" - ascending sort
#return 1 - nothing to sort
#       0 - OK
#-----------------------------------------------
function tbl_sort_arr(arr,nr,stp,  i,j,v){
  if(nr<=1) return 1;
  for(i=1; i<nr; i++)
    for(j=i+1; j<=nr; j++)
      if(((stp=="DESC") && (arr[i]<arr[j])) || 
         ((stp!="DESC") && (arr[i]>arr[j])) ){
        v = arr[i];
        arr[i] = arr[j];
        arr[j] = v;
      }
  return 0;
}

#---------------- tbl_new -----------------
#            create a new table
#d_data[key] = record (now empty)
#s_key - key of table
#s_fld - fields list (f1#f2#...#fn)
#a_ind - for record numbers and serv info
#a_ind["rec_total"] - total records count (now 0)
#a_ind["fld_list"]  = s_fld
#a_ind["fld_total"] - total fields count
#a_ind["key_index"] - number of key position in fields list
#a_ind["f"] - number of field "f" position in fields list
#return 1 - no key in fields list 
#       0 - OK
#-------------------------------------------
function tbl_new(a_ind, a_data, s_fld, s_key,    tmp_a,i,n,k){
  delete a_ind;
  delete a_data;
  a_ind["rec_total"] = 0;
  a_ind["fld_list"] = s_fld; 
  n = split(s_fld,tmp_a,"#");
  if(n==0) return 1;
  for(i=1; i<=n; i++)
    a_ind[tmp_a[i]] = i;
  a_ind["fld_total"] = n;
  k = 0; 
  for(i=1; i<=n; i++)
    if(s_key == tmp_a[i]) k = i;
  if(k==0) return 1;
  a_ind["key_index"] = k;
  return 0;
}

#---------------- tbl_put_rec -----------------
#            put a record into the table
#s_key - key of record
#a_val - array of fields values
#a_data[s_key] = string of fields values
#return 1 - record with key "s_key" inserted
#       0 - record with key "s_key" existed and updated
#----------------------------------------------
function tbl_put_rec(a_ind, a_data, s_key, a_val,   s_data){
  a_val[a_ind["key_index"]] = s_key;
  s_data = csv_make_rec(a_val,a_ind["fld_total"]);
  if(s_key in a_data){
    a_data[s_key] = s_data;
    return 0;
  }
  a_data[s_key] = s_data;
  a_ind[++a_ind["rec_total"]] = s_key;
  return 1;
}

#---------------- tbl_get_rec -----------------
#            get a record from the table
#s_key - key of record
#a_val - array of fields values
#a_data[s_key] = string of fields values
#return 1 - record with key "s_key" new, only key value put into a_val
#       0 - record with key "s_key" existed, fields values put into a_val
#----------------------------------------------
function tbl_get_rec(a_ind, a_data, s_key, a_val,   i){
  if(s_key in a_data){
    csv_get_rec(a_data[s_key],a_val);
    return 0;
  }
  delete a_val;
  for(i=1; i<=a_ind["fld_total"]; i++) a_val[i] = "";
  a_val[a_ind["key_index"]] = s_key;
  return 1;
}

#---------------- tbl_read -----------------
#            read a table from file
# 1th record in file must be a csv string with fields list
#            new table created !!!
#fn - file name
#s_key - name of key
#return 1 - no key in fields list 
#       0 - OK
#-------------------------------------------
function tbl_read(a_ind, a_data, s_key, fn,    s,tmp_a,i,n,r){
  r = getline s < fn;
  if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
  s = csv_newsep(s ,"#");
  if(tbl_new(a_ind, a_data, s, s_key)) return 1;
  while(r = getline s < fn){
    if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
    n = csv_get_rec(s,tmp_a);
    if(n < a_ind["key_index"]) return 1;
    tbl_put_rec(a_ind, a_data, tmp_a[a_ind["key_index"]], tmp_a);
  }
  close(fn);
  return 0;
}

#---------------- tbl_print -----------------
#            print a table to file
# 1th record in file is csv string with fields list
#fn - file name (if fn=="" - print to output stream)
#return - nothing
#--------------------------------------------
function tbl_print(a_ind, a_data,fn,    n,tmp_a,s){
  n = split(a_ind["fld_list"],tmp_a,"#");
  s = csv_make_rec(tmp_a,n);
  if(fn != "") print s > fn;
          else print s;
  for(n=1; n<=a_ind["rec_total"]; n++)
    if(fn != "") print a_data[a_ind[n]] > fn;
            else print a_data[a_ind[n]];
}

#---------------- tbl_sort_fld -----------------
#      sort information in an individual field 
#  field must contain subfields separated with string sep
#                example (sep = "#"):    
#    2011-03-29 52#2011-03-29 51#2009-03-30 85
#s_fld - field name
#sep   - separator
#return - nothing
#-----------------------------------------------
function tbl_sort_fld(a_ind,a_data,s_fld,sep,tp,    tmp_a,n,s,i){
  for(i=1; i<=a_ind["rec_total"]; i++){
    tbl_get_rec(a_ind,a_data,a_ind[i],tmp_a);
    s = tmp_a[a_ind[s_fld]];
    n = split(s,arr,sep);
    tbl_sort_arr(arr,n,tp);
    s = arr[1];
    for(j=2; j<=n; j++) s = s sep arr[j];
    tmp_a[a_ind[s_fld]] = s;
    tbl_put_rec(a_ind,a_data,a_ind[i],tmp_a);
  }
}

#---------------- tbl_sort ---------------------
#        sort information in a table 
#
#s_fld - fields list in string (separator -"#")
#s_stp - sort types in string (separator -"#")
#stp=="DESC" - descending sort
#stp!="DESC" - ascending sort
#return 1 - nothing to sort
#       0 - OK
#-----------------------------------------------
function tbl_sort(a_ind,a_data,s_fld,s_stp,    
                  pri,a_fld,a_stp,k,n,ai,aj,nr,i,j,si,sj,v){
  n = split(s_fld,a_fld,"#");
  split(s_stp,a_stp,"#");
  nr = a_ind["rec_total"];
  if(nr<=1) return 1;
  for(i=1; i<nr; i++)
    for(j=i+1; j<=nr; j++){
      tbl_get_rec(a_ind,a_data,a_ind[i],ai);
      tbl_get_rec(a_ind,a_data,a_ind[j],aj);
      pri = 0;
      for(k=1; k<=n; k++){
        si = ai[a_ind[a_fld[k]]];
        sj = aj[a_ind[a_fld[k]]];
        if(si == sj) continue;
        if(((a_stp[k]=="DESC") && (si<sj)) || 
           ((a_stp[k]!="DESC") && (si>sj)) )
          pri = 1;
        break;
      }
      if(pri){
        v = a_ind[i];
        a_ind[i] = a_ind[j];
        a_ind[j] = v;
      }
    }
  return 0;
}

#---------------- tbl_debug_print -----------------
#            debug print a table to file
#fn - file name (if fn=="" - print to output stream)
#return - nothing
#--------------------------------------------------
function tbl_debug_print(a_ind, a_data,    i){
  for(i in a_ind)
    if(i !~ /^[0-9]/)
      if (fn != "") printf("a_ind[%s]=%s\n",i,a_ind[i]) > fn
               else printf("a_ind[%s]=%s\n",i,a_ind[i]);
  if (fn != "") print "-------------------------"  > fn
           else print "-------------------------";
  for(i=1; i<=a_ind["rec_total"]; i++)
    if (fn != "") printf("[%s]=%s ->%s\n",i,a_ind[i],a_data[a_ind[i]]) > fn
             else printf("[%s]=%s ->%s\n",i,a_ind[i],a_data[a_ind[i]]);
}


#--------------------- HTML_spec  -----------------
#        protect specific html symbols in str
#return - new string
#--------------------------------------------------
function HTML_spec(str,   s){
  s = str;
  gsub(/\&/,"\\\\\\&amp;",s);
  gsub(/\"/,"\\\\\\&quot;",s);
  gsub(/</,"\\\\\\&lt;",s);
  gsub(/>/,"\\\\\\&gt;",s);
  return s;
}

#--------------------- read_ptn  ------------------
#              read pattern for report
#fn - file name
#return -1 if error
#--------------------------------------------------
function read_ptn(fn,  i,r,s){
  ptn_count = 0;
  while(r = getline s < fn){
    if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
    if(s ~ /^<!##########/)
      ptn_count++;
    else
      ptn_src[ptn_count] = ptn_src[ptn_count] s "\n";
  }
  close(fn);
  ptn_count++;
  for(i=0; i<ptn_count; i++)
    ptn_cur[i] = ptn_src[i];
}

#--------------------- read_strings  ------------------
#              read strings list from file
#fn - file name
#srt_arr - array for strings 
#return -1 if error
#------------------------------------------------------
function read_strings(fn,srt_arr,    s_key,s_val,r){
  while(r = getline s_key < fn){
    if(r == -1){
      print "ERROR: " fn " - " ERRNO;
      return -1;
    }
    r = getline s_val < fn
    srt_arr[s_key] = s_val;
  }
  return 0;
}

BEGIN{
  gbl_tch_all = 0;
  gbl_tch_g   = 0;
  gbl_cr_all  = 0;
  gbl_cr_g    = 0;

#--- "ID#PARENT#NAME#PATH#DEPTH#ISKAF#TCH_ALL#TCH_G#CR_ALL#CR_G"
  tbl_new(indKAF,dataKAF,"ID#PARENT#NAME#PATH#DEPTH#ISKAF#TCH_ALL#TCH_G#CR_ALL#CR_G","ID");
  
#--- "ID","PARENT","NAME","DEPTH","PATH","NUM CR"
  tbl_read(indSRV,dataSRV,"ID","baza/cr_cat.csv");
  for(i=1; i<=indSRV["rec_total"]; i++){
    cat_id = indSRV[i];
    tbl_get_rec(indKAF,dataKAF,cat_id,tmp_kaf);
    tbl_get_rec(indSRV,dataSRV,cat_id,tmp_a);
    tmp_kaf[indKAF["PARENT"]]  = tmp_a[indSRV["PARENT"]];
    tmp_kaf[indKAF["NAME"]]    = tmp_a[indSRV["NAME"]];
    tmp_kaf[indKAF["PATH"]]    = tmp_a[indSRV["PATH"]];
    tmp_kaf[indKAF["DEPTH"]]    = tmp_a[indSRV["DEPTH"]];
    tmp_kaf[indKAF["ISKAF"]]   = 0;
    tmp_kaf[indKAF["TCH_ALL"]] = 0;
    tmp_kaf[indKAF["TCH_G"]]   = 0;
    tmp_kaf[indKAF["CR_ALL"]]  = 0;
    tmp_kaf[indKAF["CR_G"]]    = 0;
    tbl_put_rec(indKAF,dataKAF,cat_id,tmp_kaf);
  }

#--- "US_ID","CR_ALL","CR_G","US_SRT","US_CAT","US_NM","US_CR"
  tbl_read(indSRV,dataSRV,"US_ID","out_tch.csv");

  for(i=1; i<=indSRV["rec_total"]; i++){
    gbl_tch_all++;
    us_id = indSRV[i];
    tbl_get_rec(indSRV,dataSRV,us_id,tmp_a);
    cat_id = tmp_a[indSRV["US_CAT"]];
    cr_g   = tmp_a[indSRV["CR_G"]];
    tch_g = 0;
    if(cr_g > 0) tch_g = 1;
    gbl_tch_g += tch_g;
    tbl_get_rec(indKAF,dataKAF,cat_id,tmp_kaf);
    np = split(tmp_kaf[indKAF["PATH"]],a_path,"/");
    for(j=2; j <=np; j++){
      tbl_get_rec(indKAF,dataKAF,a_path[j],tmp_kaf);
      tmp_kaf[indKAF["TCH_G"]] += tch_g;
      tmp_kaf[indKAF["TCH_ALL"]]++;
      tbl_put_rec(indKAF,dataKAF,a_path[j],tmp_kaf);
    }
  }

#--- "ID","VS","CT ID","FNM","SNM"
  tbl_read(indCR,dataCR,"ID","man_course/cr.csv");

# CR_TP","CR_ID","CR_LA","CR_US","CR_ACT","CR_NM","CR_AD"
  tbl_read(indSRV,dataSRV,"CR_ID","man_course/cr_info_fin.csv");
  
  for(i=1; i<=indCR["rec_total"]; i++){
    gbl_cr_all++;
    cr_id = indCR[i];
    tbl_get_rec(indCR,dataCR,cr_id,tmp_cr);
    cat_id = tmp_cr[indCR["CT ID"]];
    tbl_get_rec(indSRV,dataSRV,cr_id,tmp_a);
    cr_g = 0;
    cr_tp = tmp_a[indSRV["CR_TP"]];
    if(cr_tp > 0) cr_g = 1;
    gbl_cr_g += cr_g;
    tbl_get_rec(indKAF,dataKAF,cat_id,tmp_kaf);
    tmp_kaf[indKAF["ISKAF"]] = 1;
    tbl_put_rec(indKAF,dataKAF,cat_id,tmp_kaf);
    np = split(tmp_kaf[indKAF["PATH"]],a_path,"/");
    for(j=2; j <=np; j++){
      tbl_get_rec(indKAF,dataKAF,a_path[j],tmp_kaf);
      tmp_kaf[indKAF["CR_G"]] += cr_g;
      tmp_kaf[indKAF["CR_ALL"]]++;
      tbl_put_rec(indKAF,dataKAF,a_path[j],tmp_kaf);
    }
  }

  tbl_sort(indKAF,dataKAF,"PATH","ASC");

  npp = 1;
  fn = "_html/0.html";
  read_strings("___str.___",GB_STR);
  read_ptn("_site_ptn/0.__h");
  printf(ptn_cur[0]) > fn;

  ptn_cur[1] = ptn_src[1];
  cat_nm  = "<b>" GB_STR["MSG_ALL_AK"] "</b>"; 
  tch_all = gbl_tch_all;
  tch_g   = gbl_tch_g;
  cr_all  = gbl_cr_all;
  cr_g    = gbl_cr_g;
  gsub(/==NN==/,npp,ptn_cur[1]);
  gsub(/==CAT_NM==/,cat_nm,ptn_cur[1]);
  gsub(/==TCH_ALL==/,tch_all,ptn_cur[1]);
  gsub(/==TCH_G==/,tch_g,ptn_cur[1]);
  gsub(/==CR_ALL==/,cr_all,ptn_cur[1]);
  gsub(/==CR_G==/,cr_g,ptn_cur[1]);
  printf(ptn_cur[1]) > fn;
     
#--- "ID#PARENT#NAME#PATH#DEPTH#ISKAF#TCH_ALL#TCH_G#CR_ALL#CR_G"  
  for(i=1; i<=indKAF["rec_total"]; i++){
    cat_id = indKAF[i];
    if(cat_id == 0)continue;
    tbl_get_rec(indKAF,dataKAF,cat_id,tmp_kaf);
    tmp_kaf[indKAF["PARENT"]];
    _cat_nm  = tmp_kaf[indKAF["NAME"]];
    iskaf    = tmp_kaf[indKAF["ISKAF"]];
    _tch_all = tmp_kaf[indKAF["TCH_ALL"]];
    _tch_g   = tmp_kaf[indKAF["TCH_G"]];
    _cr_all  = tmp_kaf[indKAF["CR_ALL"]];
    _cr_g    = tmp_kaf[indKAF["CR_G"]];
    if(iskaf == 0){
      cat_nm  = "<b>" _cat_nm "</b>";
      tch_all = "<b>" _tch_all "</b>";
      tch_g   = "<b>" _tch_g "</b>";
      cr_all  = "<b>" _cr_all "</b>";
      cr_g    = "<b>" _cr_g "</b>";
    } 
    else{
      cat_nm = sprintf("<a href=\"kc%4.4d.html\">%s</a>",cat_id,_cat_nm);
      if(_tch_all != 0)
        tch_all = sprintf("<a href=\"kt%4.4d.html\">%s</a>",cat_id,_tch_all)
      else tch_all = _tch_all;
      tch_g   = _tch_g;
      cr_all  = sprintf("<a href=\"kc%4.4d.html\">%s</a>",cat_id,_cr_all);
      cr_g    =  _cr_g;
    }
    dp = tmp_kaf[indKAF["DEPTH"]]
    for(j=1; j<dp; j++) cat_nm = " ___ " cat_nm;
    ptn_cur[1] = ptn_src[1];
    gsub(/==NN==/,npp++,ptn_cur[1]);
    gsub(/==CAT_NM==/,cat_nm,ptn_cur[1]);
    gsub(/==TCH_ALL==/,tch_all,ptn_cur[1]);
    gsub(/==TCH_G==/,tch_g,ptn_cur[1]);
    gsub(/==CR_ALL==/,cr_all,ptn_cur[1]);
    gsub(/==CR_G==/,cr_g,ptn_cur[1]);
    printf(ptn_cur[1]) > fn;
  }
   
   printf(ptn_cur[2]) > fn;

#  tbl_print(indKAF,dataKAF,"out_kaf.csv");

#  tbl_debug_print(indKAF,dataKAF);
}
